---
jupytext:
  formats: md:myst
  text_representation:
    extension: .md
    format_name: myst
    format_version: 0.13
    jupytext_version: 1.11.5
kernelspec:
  display_name: Python 3 (ipykernel)
  language: python
  name: python3
---

# regex über ein dataframe

* J.Busse, www.jbusse.de, 2022-04-19

Lizenz: public domain / [CC 0](https://creativecommons.org/publicdomain/zero/1.0/deed.de)

+++

Aufgabenstellung: 
* Gegeben ist ein Liste von Dateinamen. Klassifiziere die Dateinamen nach der Endung!

Nebenbedingung: Um nachfolgende Textmining-Schritte gut vorzubereiten, wollen wir diese und viele weitere Analyse-Ergebnisse in einer Tabelle, in Python also einem Pandas-Dataframe festhalten.

```{code-cell} ipython3
import numpy as np
import pandas as pd
import re 
import glob
```

## Dateien anzeigen

Was in Linux ein `ls` wäre, geht in Python so:

```{code-cell} ipython3
# filename_list = glob.glob("mm/*") 

filename_list = [ "./data/mm/jb1.mm", "../home/hallo.py"] 
print(filename_list)
```

## Regex für Pfad, Name, Extension

**Aufgabe:** Schreibe eine Funktion `pne()`, die Pfad, Name und Extension ohne die entsprechenden Trennzeichen als Tupel zurückgibt:

* Eingabe: `pne("mm/jb-1-1.mm")`
* Rückgabe: `("mm", "jb-1-1", "mm")`

Spielwiese: <https://regex101.com/>

Hinweis: Natürlich gibt es diese Funktion auch in einer Bibliothek. Aber wir wollen ja regex ausprobieren, richtig?

```{code-cell} ipython3
def pne(filename):
    # suboptimale Lösung: mit regex besser machen!
    # return filename[0:2], filename[4:-3], filename[-2:]
    
    regex = r"(.+)\/(.+)\.(.+)"
    m_search = re.search(regex, filename)
    return m_search[1], m_search[2], m_search[3]
```

```{code-cell} ipython3
pne("ein/pfad/datei.md")
```

```{code-cell} ipython3
regex = r"(.+)\/(.+)\.(.+)"
test_str = ("ein/pfad/datei.md\n./data/md/hallo.pdf")
match = re.findall(regex, test_str)
match

m_search = re.search(regex, test_str)
m_search[3]
```

Damit kann man nun typischerweise mit List-Comprehension eine neue Liste erzeugen:

```{code-cell} ipython3
filename_pne = [ pne(f) for f in filename_list ]
print(filename_pne)
```

Oder ein schickes Dictionary:

```{code-cell} ipython3
filename_dict = { f: pne(f) for f in filename_list }
print(filename_dict)
```

## Pandas

+++

Mit [pandas.DataFrame.from_dict](https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.from_dict.html) lässt sich solcg ein dict elegant in ein Pandas Dataframe überführen:

```{code-cell} ipython3
dateien_df = pd.DataFrame.from_dict(filename_dict, 
                    orient='index', 
                    columns = ["path", "name", "ext"])
dateien_df.head()
```

```{code-cell} ipython3

```

```{code-cell} ipython3

```
